/**
 * @module multiparfiles
 */
/** */
import {$log} from "ts-log-debug";
import {EndpointMetadata} from "../../mvc/class/EndpointMetadata";
import {EndpointInfo, Middleware, Next, Req, Res} from "../../mvc/decorators";
import {IMiddleware} from "../../mvc/interfaces";
import {ServerSettingsService} from "../../server/services/ServerSettingsService";
/**
 * @private
 * @middleware
 */
@Middleware()
export class MultipartFileMiddleware implements IMiddleware {

    private multer;

    constructor(private serverSettingsService: ServerSettingsService) {

        try {
            /* istanbul ignore else */
            if (require.resolve("multer")) {
                this.multer = require("multer");
            }
        } catch (er) {

        }
    }

    /**
     *
     * @param endpoint
     * @param request
     * @param response
     * @param next
     * @returns {any}
     */
    use(@EndpointInfo() endpoint: EndpointMetadata,
        @Req() request: Express.Request,
        @Res() response: Express.Response,
        @Next() next) {

        if (this.multer) {
            const options = Object.assign({
                dest: this.serverSettingsService.uploadDir
            }, endpoint.store.get(MultipartFileMiddleware) || {});

            const middleware = this.multer(options);

            return middleware.any()(request, response, next);
        } else {
            $log.warn("Multer isn't installed ! Run npm install --save multer before using Multipart decorators.");
        }

    }
}